<?php 

/**
	 MPD Administration Controller
	
	 @Copyright 2015 Stefan Rick
	 @author Stefan Rick
	 Mail: stefan@rick-software.de
     Web: http://www.netzberater.de
		
 */


class Setup extends Service {
	protected $pname = 'mpd';
	public $viewname = 'Music Player MPD';
	public $mpdconfig = array('music_directory' => '', 'playlist_directory' => '');
	public $mpdconfigoutput = array('type' => 'alsa', 'name' => 'max2play','device' => '', 'format' => '', 'mixer_device' => '', 'mixer_control' => '', 'audio_buffer_size' => '', 'buffer_before_play' => '' );
	public $ympdconfig = array();
	
	public function __construct(){								
		parent::__construct();
		$this->pluginname = _('Music Player MPD');
		
		$this->registerLocale(dirname(__FILE__).'/../locale', 'mpd');
		$this->scriptPath = dirname(__FILE__).'/../scripts/';
		$this->configFile = '/etc/mpd.conf';
		
		if($_REQUEST['ajax'] == 1 && $_REQUEST['action'] == 'install'){
			//Function to get Progress of Installation
			$this->_install(1);
			ob_end_clean();
			echo implode('<br />', $this->view->message);
			ob_flush();
			die();
		}
		
		$this->_checkinstall();
		$this->_getMPDConfig();
		$this->getSoundDevices();
		
		if(isset($_REQUEST['action'])){
			if($_REQUEST['action'] == 'startmpd'){			
				$this->view->message[] = $this->start($this->pname, '', '', true);			
			}
			
			if($_REQUEST['action'] == 'stopmpd'){			
				$this->view->message[] = $this->stop($this->pname, '', '', true);			
			}
			
			if($_REQUEST['action'] == 'killmpd'){
				$this->view->message[] = $this->kill($this->pname);
			}
			
			if($_REQUEST['action'] == 'save_mpdconfig'){
				$this->_saveMPDConfig();				
			}
			
			if($_REQUEST['action'] == 'install'){
				$this->_install();
			}
			if($_REQUEST['action'] == 'resetEqualizer'){
				$this->updateEqualizer(0, 'mpd');				
			}
			if($_REQUEST['action'] == 'saveEqualizer'){
				$this->updateEqualizer($_REQUEST['settingsEqualizer'], 'mpd');
			}
		}
		$this->view->pid = $this->status($this->pname);
		$this->showHelpSidebar();
		$this->getEqualizer('mpd');
		$this->getDebug();
	}
	
	/**
	 * Save all config Parameters for MPD
	 */
	private function _saveMPDConfig(){
		$this->selectAutostart(isset($_REQUEST['autostartmpd']) ? 1 : 0, true, $this->pname);
		$this->selectAutostart(isset($_REQUEST['autostartympd']) ? 1 : 0, true, 'ympd');
		
		foreach($this->mpdconfig as $key => $value){
			if($_REQUEST['mpd'][$key] != '' && $_REQUEST['mpd'][$key] != $this->mpdconfig[$key]){
				$this->saveConfigFileParameter($this->configFile, $key, '\"'.$_REQUEST['mpd'][$key].'\"', '\\t');								
			}
		}
		$save = false;
		foreach($this->mpdconfigoutput as $key => $value){
			if(isset($_REQUEST['mpdoutput'][$key]) && $_REQUEST['mpdoutput'][$key] != $this->mpdconfigoutput[$key]){
				//Build config and Replace existing one
				$this->mpdconfigoutput[$key] = $_REQUEST['mpdoutput'][$key];
				$save = true;				
			}
		}
		if($save){
			$output = '';
			foreach($this->mpdconfigoutput as $key => $value){
				if($value != '')
					$output.= '\n\t'.$key.'\t\"'.$value.'\"';
			}
			$this->writeDynamicScript(array('awk \'BEGIN{RS="\r\n"} sub(/\naudio_output {[^}]*/, "\naudio_output {'.$output.'\n")\' /etc/mpd.conf > /etc/mpd.tmp.conf && mv /etc/mpd.tmp.conf /etc/mpd.conf'));
		}
		
		//set Mixer Device by choosen sound device and override previous setting
		if(isset($this->view->soundDevices[$_REQUEST['mpd']['device']]['cardnumber']) && strlen($this->view->soundDevices[$_REQUEST['mpd']['device']]['cardnumber']) > 0){
			$this->saveConfigFileParameter($this->configFile, 'mixer_device', '\"hw:'.$this->view->soundDevices[$_REQUEST['mpd']['device']]['cardnumber'].'\"', '\\t');
		}
		
		// set Port for YMPD --webport [PORT]
		if($_REQUEST['portympd'] != '' && $_REQUEST['portympd'] != $this->view->portympd){
			$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'YMPD_PARAMETER', '--webport '.intval($_REQUEST['portympd']));
		}
		
		// Simultanous Use of Kodi and MPD?
		$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'USE_USB_DAC', intval($_REQUEST['use_usb_dac']));
		
		$this->_getMPDConfig();
		return true;
	}
	
	private function _getMPDConfig(){
		$this->view->autostartmpd = $this->checkAutostart($this->pname, true);
		$this->view->autostartympd = $this->checkAutostart('ympd', true);
		
		//New Version for Multiple Audio-Outputs		
		$output = trim($this->writeDynamicScript(array("awk '/^audio_output/,/}/' /etc/mpd.conf")));
		/**
		 audio_output {
			type	"alsa"
			name	"Max2Play ALSA Device"
			device	"sysdefault:CARD=sndrpijustboomd"
			mixer_type	"hardware"
			mixer_device	"hw:1"
			mixer_control	"Digital"
			format	"1920000:24:2"
		 */
		if($output){
			if(preg_match_all('=^[\t ]*([^ \t]+)[ \t]+"([^"]+)=im', $output, $matches)){
				// $matches[1] parameter
				// $matches[2] values
				$parameter = array_flip($matches[1]);
				foreach($this->mpdconfigoutput as $key => $value){					
					if(isset($parameter[$key]))
						$this->mpdconfigoutput[$key] = $matches[2][$parameter[$key]];
				}
			}
			
		}
		foreach($this->mpdconfig as $key => $value){
			$this->mpdconfig[$key] = str_replace('"','',$this->getConfigFileParameter($this->configFile, $key, '[\\t ]*'));
		}		
		
		$this->view->use_usb_dac = $this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'USE_USB_DAC');
		
		$this->view->portympd = str_replace('--webport ','',$this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'YMPD_PARAMETER'));
		return true;
	}
	
	private function _install($ajax = 0){						
		if($this->view->installed){
			$this->view->message[] = _t('Already installed');
			return false;
		}
		
		if($ajax == 0){
			if($this->getProgressWithAjax('/opt/max2play/cache/install_mpd.txt', 1, 1, 25)){
				// Run installer as Deamon
				sleep(1);
				$shellanswer = $this->writeDynamicScript(array($this->scriptPath."install_mpd.sh ".$this->scriptPath." >> /opt/max2play/cache/install_mpd.txt 2>&1 &"), false, true);
			}
		}else{
			$status = $this->getProgressWithAjax('/opt/max2play/cache/install_mpd.txt', 0, 0, 25);
			$this->view->message[] = nl2br($status);
			if(strpos($status, 'finished') !== FALSE){
				//Finished Progress - did not delete progressfile yet
				$this->view->message[] = _t('Music Player MPD is now installed!');
				shell_exec('rm /opt/max2play/cache/install_mpd.txt');
				$this->selectAutostart( 1, true, 'ympd');
				$this->selectAutostart( 1, true, 'mpd');
				$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'YMPD_PARAMETER', '--webport 8081');
			}
		}
		return true;
	}
	
	private function _checkinstall(){
		$this->view->installed = trim(shell_exec('if [ -e "/etc/init.d/mpd" ]; then echo "1";else echo "0"; fi;'));
				
		//Check for autostart and running Squeezelite / Shairport / Kodi and print Message
		$autosqueezelite = $this->checkAutostart('squeezelite', true);
		$autoshairport = $this->checkAutostart('shairport', true);
		
		if($autosqueezelite || $autoshairport){
			$this->view->message[] = _t('<b>Important:</b> Squeezelite / Shairport Audioplayer is set to autostart. This might conflict with MPD Music Player (no sound output). It is recommended to remove the Autostart setting for all Audioplayers on tab Audioplayer and stop them.');
		}
		return true;
	}
	
	/**
	 * get Playable Sounddevices for MPD from Squeezelite
	 * Style:
	 * default:CARD=ALSA              - bcm2835 ALSA, bcm2835 ALSA - Default Audio Device
	 */
	public function getSoundDevices(){
		$this->soundDeviceLog = $this->writeDynamicScript(array('/opt/squeezelite/squeezelite -l'));
		$aplayOutput = $this->writeDynamicScript(array('aplay -l | grep "card"'));
		//card 0: ALSA [bcm2835 ALSA], device 1: bcm2835 ALSA [bcm2835 IEC958/HDMI]
		//card 1: IQaudIODAC [IQaudIODAC], device 0: IQaudIO DAC HiFi pcm512x-hifi-0 []
		preg_match_all('=^card ([0-9]): ([^ ]*)=im', $aplayOutput, $aplayMatches);
		$count = count($aplayMatches[1]);
		$aplayCards = array();
		for($i = 0; $i < $count; $i++){
			$aplayCards[$aplayMatches[2][$i]] = $aplayMatches[1][$i];
		}
		
		//Regex zerlegung:
		// $matches[1][x] = Name
		// $matches[2][x] = Karte/Device
		// $matches[3][x] = Beschreibung
		// $matches[4][x] = Name ist PCM Software Device
		// $matches[5][x] = Beschreibung von PCM Software Device
		// $matches[6][x] = PCM Software Device ohne Beschreibung
		preg_match_all('=^[ ]*([^:\n]+)[ ]*:[ ]*([^-\n ]+)[ ]*-[ ]*(.*)[ ]*|^[ ]*([^\n- ]+)[ ]*-[ ]*(.*)[ ]*|^[ ]*([^\n-:]+)[ ]*$=im', $this->soundDeviceLog, $matches);
		$count = count($matches[1]);
		for($i = 0; $i < $count; $i++){
			$dev = array();
			
			// get card position (hw:0 / hw:1)
			$plaincardname = preg_replace('@.*CARD=|,.*@', '', $matches[2][$i]);
			$dev['cardnumber'] = isset($aplayCards[$plaincardname]) ? $aplayCards[$plaincardname] : '';
			
			//Fallunterscheidung je nach Devices (Software / Hardware / mit und ohne Hinweistext)
			if($matches[1][$i] !== ''){
				$dev['name'] = $matches[1][$i];
				$dev['card'] = $matches[2][$i];
				$dev['description'] = $matches[3][$i];
				$this->view->soundDevices[$matches[1][$i].':'.$matches[2][$i]] = $dev;
			}elseif($matches[4][$i] !== ''){
				$dev['name'] = $matches[4][$i];
				$dev['card'] = '';
				$dev['description'] = $matches[5][$i];
				//add without "plug:"
				if($matches[4][$i] == 'bluetoothspeaker' || $matches[4][$i] == 'pulse')
					$this->view->soundDevices[$matches[4][$i]] = $dev;
				else
					$this->view->soundDevices['plug:'.$matches[4][$i]] = $dev;
			}elseif($matches[6][$i] !== ''){
				$dev['name'] = $matches[6][$i];
				$dev['card'] = '';
				$dev['description'] = $matches[6][$i];
				$this->view->soundDevices['plug:'.$matches[6][$i]] = $dev;
			}			
		}
		unset($this->view->soundDevices['plug:null']);
	
		return true;
	}
	
	private function getDebug(){
		$out = array();
		$out['MPD Version'] = shell_exec('dpkg -s mpd | grep Version');
		//$out['MPD Config'] = shell_exec('cat /etc/mpd.conf');
		$out['AUDIO-DEVICES (aplay -L)'] = $this->writeDynamicScript(array('aplay -L'));
		$out['MPD-LOGFILE LAST 50 ENTRIES'] = shell_exec('tail -50 /var/log/mpd/mpd.log');
		$this->view->debug = $out;
		return true;
	}
	
	public function showHelpSidebar(){
		global $helpSidebar;
		$helpSidebar['title'] = _t('Help - Music Player');
		$helpSidebar['content'] = _t('<ul><li>Watch <a target="_blank" href="https://www.youtube.com/channel/UCu2JHGI142fLrkoUN8dUkRA">a video tutorial about the settings on this page</a> and use different possibilities of listening to your music.</li><li><b>Connecting a USB-Drive</b>: just attach your USB-storage and you find its content in the music folder of the musicplayer.</li><li><b>Accessing the storage on this device</b>: use the Samba-Share and connect from any computer (file explorer) in your network with \\\\max2play</li><li><b>Access a network storage with your music</b> (e.g. Diskstation): Mount the share to the path /mnt/mpd (using filesystem mount) and you will find it in your music folder.</li></ul>');
		return true;
	}
}

$mpd = new Setup();

include_once 'Ampache.php';

include_once(dirname(__FILE__).'/../view/setup.php');



